// stdafx.cpp : source file that includes just the standard includes
//	PunchAcard.pch will be the pre-compiled header
//	stdafx.obj will contain the pre-compiled type information

#include "stdafx.h"

/* punch processor for punch card file
// data is read in a byte at a time from (stdin)
// the data is placed in an 80 byte buffer
// if there is more than 80 bytes, an error is declared
// the end of a card is defined by a line feed character (lf)
// trailing spaces are scanned off; the card can be 0 to 80 bytes
// each byte is checked for validity.
// if there is a character that does not translate to a keypunch code,
//     an error is declared.
// the data from the buffer is written to (stdout) with a line feed (lf)
//     following each card record
// errors will be printed on (stderr)
// the code will return the number of errors found.
// a return code of zero indicated valid processing.
*/

#include <stdio.h>
#include <conio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <errno.h>
#include "RmtPunch.h"
#include "cbw.h"

#define CHM

int		main(int argc, char * argv[]);
int		getCardASCII(void);
int		getCardBinary(void);
int		setTable(void);
void	punchThemCards();
int		CommAccess(int,int);
int     checkFileType(char *);
int		relayRattle();

int		inchar;
unsigned short	cardBuffer[82];
unsigned short 	KeyCode[256]; /*Place to store all of the key codes*/
int		status;
int		i;
int     fileType;
int		returnStatus=0;
int		errorCount=0;
int		cardCount=0;
int		inColCount=0;
int		outColCount=0;
int		nonSpace;
char	passedFileName[200]="C:\\don.tmp";
char	tempFileName[200];
char	inFileName[200];
char	outFileName[200];
char	tempName[200];
int		cardsToSkip=0;
int		skipCards=FALSE;

char    findFile[]="c:\\RmtPunch\\OpenDiag.exe";
FILE 	*in, *out;
unsigned short     dummy=0;
relayCntl(int ,int , unsigned short *);


int main(int argc, char * argv[])

{
	
	fprintf(stderr,"New Remote Card Punch 09-19-2014  started\n\n");
	
	// set the table of ASCII to card punches
	status=setTable();
	
	// open the com1: line
	status=0;
	fprintf(stderr,"Opening RS-232 communication line for R25 control\n");

	status=CommAccess(OPEN,0);

	if(status !=0)
	{
		fprintf(stderr,"CommAccess OPEN failed with a return code of %d\n",status);
			returnStatus+=2;
		goto exit;
	}
	
	// get the file name and path to process and place it in c:\don.tmp 
	i=system(findFile);	
	if(i != 0)
	{
		fprintf(stderr,"Unable to run %s\nProgram terminated\n",findFile);
		fprintf(stderr,"Press any key to exit......");
		getch();
		returnStatus+=4;
		goto exit;
	}
	
	// read in the path and file name
	if ((in = fopen(passedFileName, "r")) == NULL)
	{
		fprintf(stderr, "Cannot open input file %s.\n",
		       passedFileName);
		fprintf(stderr,"Press any key to exit......");
		getch();
		returnStatus+=8;
		goto exit;
	}
		i=fread(inFileName,1,64,in); //read in the drive,path and file name left for you
	if(i < 3)
	{
		fprintf(stderr,"Error reading file %s\n",passedFileName);
		fprintf(stderr,"Press any key to exit......");
		getch();
		returnStatus+=16;
		goto exit;
	}
	fclose(in); // close c:\don.tmp

	inFileName[i]=0;      //add trailing null to define string
	i=strlen(inFileName);
	sprintf(tempName,"%s",inFileName);
	
	// build the name of the output file
	
	sprintf(outFileName,"%s",passedFileName);
	
	fileType=checkFileType(inFileName);
	printf("\nFILE TYPE = %d\n",fileType);
	
	if(fileType == ASCII)
	{
		printf("\n Processing ASCII input file %s\n",inFileName);
		if ((in = fopen(inFileName, "r")) == NULL)
		{
			fprintf(stderr, "Cannot open input file %s.\n",
				 inFileName);
			fprintf(stderr,"Press any key to exit......");
			getch();
			returnStatus+=32;
			goto exit;
		}
	}
	else
	{
		printf("\n Processing Binary input file %s\n",inFileName);
		if ((in = fopen(inFileName, "rb")) == NULL)
		{
			fprintf(stderr, "Cannot open input file %s.\n",
				 inFileName);
			fprintf(stderr,"Press any key to exit......");
			getch();
			returnStatus+=32;
			goto exit;
		}
	}



	if ((out = fopen(outFileName, "wb")) == NULL)
	{
		fprintf(stderr, "Cannot open output file %s\n",
		       outFileName);
		fprintf(stderr,"Press any key to exit......");
		getch();
		returnStatus+=64;
		goto exit;
	}
	
	fprintf(stderr,"\nProcessing input file %s\n",inFileName);

	for(EVER)
	{
		if(fileType == ASCII)
		{
			status=getCardASCII();
		}
		else
		{
			status=getCardBinary();
		}
		if(status == EOF) break;
		if(status < 0)
		{
			returnStatus=64;
			goto exit;
		}
	}

	fclose(in);
	fclose(out);
    

    if(errorCount == 0)
	{
		fprintf(stderr,"Cards to be punched \t\t%ld\n",cardCount);
		fprintf(stderr,"Columns of data read in \t%ld\n",inColCount);
		fprintf(stderr,"Columns of data to be punched \t%ld\n",outColCount);
		
		for(EVER)
		{
#ifdef CHM
			fprintf(stderr,"\nMake sure the keypunch has been on for a minute then:\n");
			fprintf(stderr,"\tPress 'P' to punch\n");
			fprintf(stderr,"\tPress 'S' to skip cards and then punch\n");
			fprintf(stderr,"\tPress 'T' to test relays\n");
#endif
			fprintf(stderr,"\tPress 'E' to exit:");

			i=getch();
#ifdef CHM
			if((i=='P') || (i=='p'))
			{
				fprintf(stderr,"\n\nPress any key to pause or exit the punching\n");
				punchThemCards();
				break;
			}
			if((i=='S') || (i=='s'))
			{
			printf("cards to skip = %d\n",cardsToSkip);	fprintf(stderr,"\n\nEnter the number of cards to skip:> ");
				fscanf(stdin,"%d",&cardsToSkip);
				if(cardsToSkip > (cardCount-1)) 
				{
					fprintf(stderr,"\nYou don't have that many\n");	
				}
				else
				{
					printf("cards to skip = %d\n",cardsToSkip);punchThemCards(); // again, again, and again
					break;
				}
				printf("cards to skip = %d\n",cardsToSkip);
			}
#endif
			if((i=='E') || (i=='e'))
			{
				break;
			}

#ifdef CHM	
			if((i=='T') || (i=='t'))
			{
				status=0;
				status=relayRattle();
				if(status != 0)
				{
					printf("\n\nError returned from test relay command\n");
					fprintf(stderr,"Press any key to exit......");
					getch();
					break;
				}
			}
#endif
		}
	}
	
	returnStatus=0;   // if everything went ok
exit:
	
	status=CommAccess(CLOSE,0);	
	
	// clear relays
	status=0;
#ifdef CHM
	status=relayCntl(CLRRELAY,0,&dummy);//  clear relays
#endif
	if(i == 0)
	{
		fprintf(stderr,"\nGot an error on relay control clear all relays\n");
		returnStatus+=256;
	
	}
	fprintf(stderr,"\n\nPunchAcard ended with a return status of %x\n\n\n",returnStatus);
    fprintf(stderr,"\n\nPress any key to close the window\n");
    i=getch();
	return(returnStatus);
}




int relayRattle()
{

	union
	{
	unsigned int all;
	unsigned char bytes[4];
	} count;

	int i;
	int status;
	unsigned short dport1;
	unsigned short dport2;
	unsigned short dport3;
	unsigned short dport4;

	count.all=0x1;
	for(i=0;i<24;i++)
	{
		dport1 = (unsigned short) count.bytes[2];
		dport2 = (unsigned short) count.bytes[1];
		dport3 = (unsigned short) count.bytes[0]>>4;
		dport4 = (unsigned short) count.bytes[0];

		status=relayCntl(SETRELAY,FIRSTPORTA,&dport1);
		if(status != 0) return(status);
		status=relayCntl(SETRELAY,FIRSTPORTB,&dport2);
		if(status != 0) return(status);
		status=relayCntl(SETRELAY,FIRSTPORTCH,&dport3);
		if(status != 0) return(status);
		status=relayCntl(SETRELAY,FIRSTPORTCL,&dport4);
		if(status != 0) return(status);

		Sleep(20);
		count.all*=2;




	}


	return(0);



}
